% Filename: HomWienerSpatial.m
%User-interactive spatially adaptive homomorphic Wiener filtering
% Aleksandra Pizurica <Aleksandra.Pizurica@telin.UGent.be> TELIN/Ghent University
% Last modified: May 2004.

clear all;
close all;

Ref=input('Reference noise-free image available? (y/n) ','s');

% Read reference, noise-free image
if (Ref=='y')
    InputType=0;
    while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
     InputType=input('\nReference image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
    end
    imload;
    Xclean=SCAN;
    clear SCAN;
    [M,N]=size(Xclean);
  
    Choice=input('Choose 1 or 2:\n1. An arbitrary noise variance\n2. A stored noisy image\n ');

    if(Choice==1)
       NoiseStd=input('Standard Deviation of added noise: ');
       Xnoisy=Xclean+randn(M,N)*NoiseStd;
       clean_image=input('Reference image name: ','s');
       image_name=strcat(clean_image,'_std',num2str(NoiseStd));
    else
       InputType=0;
       while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
         InputType=input('\nNoisy image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
       end
       imload;
       Xnoisy=SCAN;
       clear SCAN;
  
       %Extract the image name from the filename
       image_name=filename(1:(length(filename)-4))
    end
  
else % No reference noise-free image image available

    % Read noisy image
    InputType=0;
    while (InputType~=1 & InputType~=2 & InputType~=3 & InputType~=4)  
        InputType=input('\nNoisy image format:\n1. "*.tif"\n2. "*.sg2"\n3. "*.mat"\n4. "*.bin"\n>>  ');
    end
    imload;
    Xnoisy=SCAN;
    clear SCAN;

    %Extract the image name from the filename
    image_name=filename(1:(length(filename)-4))
end
 
  
%=====================================================

figure('units','normalized','position',[0.05 0.1 0.3 0.8]);
subplot(2,1,1),imagesc(abs(Xnoisy));title('Input image');
colormap(gray);
  

if(Ref=='y')
    Ps=std2(Xclean)
    ChangeWindow='y';
    while(ChangeWindow=='y')
       W=input('Window size: ');
       Xden=exp(wiener2(log(Xnoisy+1),[W,W]))-1;
       subplot(2,1,2),imagesc(Xden);
       title('Denoised image');
       Prn=std2(Xclean-Xden);
       snr_res=20*log10(Ps/Prn)
       ChangeWindow=input('Change window size (y/n): ', 's' );
    end
    Ps=std2(Xclean);
    Pn=std2(Xclean-Xnoisy);
    Prn=std2(Xclean-Xden);
    snr_input=20*log10(Ps/Pn)
    snr_res=20*log10(Ps/Prn)
    MSE=mean(mean((Xclean-Xden).^2));
    MSEin=mean(mean((Xclean-Xnoisy).^2));
    PSNR=10*log10(255^2/MSE)
    PSNRin=10*log10(255^2/MSEin);
else
    ChangeWindow='y';
    while(ChangeWindow=='y')
       W=input('Window size: ');
       Xden=exp(wiener2(log(Xnoisy+1),[W,W]))-1;
       subplot(2,1,2),imagesc(Xden);
       title('Denoised image');
       ChangeWindow=input('Change window size (y/n): ', 's' );
    end
end
    

   
%============================================
% Save result

Save_image=input('Save? y/n ','s')

if (Save_image=='y');
   Filename=strcat(image_name,'_HomWiener_',num2str(W),'x',num2str(W),'.mat');
   X=Xden;
   WindowSize=W;
   if(Ref=='y')
      save(Filename,'X','snr_input','snr_res','PSNRin','PSNR', 'MSE');
   else
      save(Filename,'X');
   end 
end

